//  
//  Copyright (C) 2010 Robert Dyer, Rico Tzschichholz
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

using Cairo;
using Gdk;
using Gtk;
using Wnck;

using Docky.CairoHelper;
using Docky.Items;
using Docky.Interface;
using Docky.Services;
using Docky.Services.Logging;
using Docky.Services.Applications;

namespace Docky.Menus
{
	public class DockFolderMenu : DockMenu
	{
		MenuCrumbsWidget crumbs;
		FolderStackWidget stack;
		VScrollbar scrollbar;
		
		public DockFolderMenu (Gtk.Window parent) : base (parent)
		{
		}
		
		public bool SetItem (FileDockItem fdi)
		{
			if (fdi.OwnedFile.Exists && fdi.OwnedFile.QueryFileType () == GLib.FileType.Directory)
				SetFolder (fdi.OwnedFile);
			else
				Console.WriteLine ("Not a directory! - {0}", fdi.OwnedFile.Path);
			
			return true;
		}
		
		void SetFolder (GLib.File folder)
		{
			Clear ();
			
			VBox vbox = new VBox ();
			Container.Add (vbox);
			
			crumbs = new MenuCrumbsWidget (folder.Dup ());
			crumbs.FolderChanged += HandleCrumbsFolderChanged;
			vbox.PackStart (crumbs);
			
			HBox hbox = new HBox ();
			vbox.PackStart (hbox);
			
			stack = new FolderStackWidget (folder.Dup ());
			stack.FolderChanged += HandleFilesFolderChanged;
			hbox.PackStart (stack);
			
			scrollbar = new VScrollbar (stack.Adjustment);
			hbox.PackEnd (scrollbar);
			
			vbox.SetSizeRequest (-1, -1);

			Container.ShowAll ();
		}

		void HandleFilesFolderChanged (object sender, EventArgs e)
		{
			if (crumbs != null && sender == stack)
				crumbs.CurrentFolder = stack.CurrentFolder;
		}

		void HandleCrumbsFolderChanged (object sender, EventArgs e)
		{
			if (stack != null && sender == crumbs)
				stack.CurrentFolder = crumbs.CurrentFolder;
		}

		void Clear ()
		{
			if (stack != null) {
				if (scrollbar != null)
					scrollbar.Dispose ();
				stack.FolderChanged -= HandleFilesFolderChanged;
				stack.Dispose ();
			}
			
			if (crumbs != null) {
				crumbs.FolderChanged -= HandleCrumbsFolderChanged;
				crumbs.Dispose ();
			}
			
			if (Container != null && Container.Child != null) {
				Gtk.Widget w = Container.Child;
				Container.Remove (w);
				w.Dispose ();
				w.Destroy ();
			}
		}
		
		protected override bool OnKeyPressEvent (EventKey evnt)
		{
			if (evnt.Key == Gdk.Key.Up) {
			} else if (evnt.Key == Gdk.Key.Down) {
			} else if (evnt.Key == Gdk.Key.Return) {
			} else if (evnt.Key == Gdk.Key.Escape) {
				Hide ();
			} else {
			}
			
			return base.OnKeyPressEvent (evnt);
		}
		
		public override void Dispose ()
		{
			Clear ();
			base.Dispose ();
		}
	}
}
