//  
//  Copyright (C) 2010 Robert Dyer, Rico Tzschichholz
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;
using System.Collections.Generic;
using System.Linq;

using Docky.CairoHelper;
using Docky.Interface;
using Docky.Services;

namespace Docky.Menus
{
	public class FolderStackWidget : StackWidget
	{
		public event EventHandler FolderChanged;
		
		int files_count;

		GLib.File currentFolder;
		public GLib.File CurrentFolder {
			get { return currentFolder; }
			set {
				if (currentFolder != null && value != null && currentFolder.Path == value.Path)
					return;
				
				if (currentFolder != null)
					currentFolder.Dispose ();
			
				if (value == null) {
					currentFolder = null;
					return;
				}
				
				currentFolder = value.Dup ();
				
				//List<GLib.File> dirs = new List<GLib.File> ();
				//List<GLib.File> files = new List<GLib.File> ();
				IEnumerable<GLib.File> dirs;
				IEnumerable<GLib.File> files;
				
				try {
					/*GLib.FileEnumerator enumerator = currentFolder.EnumerateChildren ("standard::type,standard::name,standard::is-hidden", GLib.FileQueryInfoFlags.None, null);
					if (enumerator != null) {
						GLib.FileInfo info;
						while ((info = enumerator.NextFile ()) != null) {
							if (!info.GetAttributeBoolean ("standard::is-hidden") && !info.Name.EndsWith ("~")) {
								GLib.File child = currentFolder.GetChild (info.Name);
								if (info.FileType == GLib.FileType.Directory)
									dirs.Add (child);
								else
									files.Add (child);
							}
							info.Dispose ();
						}
						enumerator.Close (null);
						enumerator.Dispose ();
					}*/
					
					dirs = currentFolder.SubDirs (false);
					files = currentFolder.GetFiles ();
					
					/*Files = currentFolder.SubDirs (false)
						.Where (d => !d.Basename.EndsWith ("~") && !d.QueryInfo<bool> ("standard::is-hidden"))
						.OrderBy (d => d.Basename)
						.Union (currentFolder.GetFiles ()
						.Where (f => !f.Basename.EndsWith ("~") && !f.QueryInfo<bool> ("standard::is-hidden"))
						.OrderBy (f => f.Basename));*/
					
					IEnumerable<GLib.File> elements = dirs.OrderBy (d => d.Basename).Union (files.OrderBy (f => f.Basename));

					// cache this value for SourecElementsCount due performance issues
					files_count = elements.Count ();
					
					IEnumerator<GLib.File> fenumerator = elements.GetEnumerator ();
					while (fenumerator.MoveNext ()) {
						AddElement (new FileStackElement (fenumerator.Current));
					}
					
				} catch (GLib.GException e) {
					Log.Error (e.Message);
					files_count = 0;
				}
				
				LayoutElements ();
				
				if (FolderChanged != null)
					FolderChanged (this, EventArgs.Empty);
				
				QueueDraw ();
			}
		}
		
		public FolderStackWidget (GLib.File folder) : base (Gdk.Screen.Default.Width / (2 * DefaultElementWidth), Gdk.Screen.Default.Height / (2 * DefaultElementHeight), StackScrollDirection.Vertical)
		{
			CurrentFolder = folder;
			IconSizeChanged += HandleIconSizeChanged;
		}

		void HandleIconSizeChanged (object sender, EventArgs e)
		{
			VisibleColumns = Gdk.Screen.Default.Width / (2 * DefaultElementWidth);
			VisibleRows = Gdk.Screen.Default.Height / (2 * DefaultElementHeight);
		}
		
		#region implemented abstract members of StackWidget
		protected override bool OnElementClicked (StackElement element)
		{
			if (element is FileStackElement) {
				if ((element as FileStackElement).OwnedFile.QueryFileType () != GLib.FileType.Directory) {
					DockServices.System.Open ((element as FileStackElement).OwnedFile);
					return false;
				} else {
					CurrentFolder = (element as FileStackElement).OwnedFile;
					return true;
				}
			}
			return true;
		}
		
		public override int SourceElementsCount ()
		{
			return files_count;
		}
		#endregion

		public override void Dispose ()
		{
			FolderChanged = null;
			CurrentFolder = null;
			IconSizeChanged -= HandleIconSizeChanged;
			base.Dispose ();
		}
	}
}
